## $Id: browser.py 10573 2013-08-31 15:02:33Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""UI components for basic applicants and related components.
"""
import grok
from zope.component import getUtility
from zope.i18n import translate
from waeup.kofa.widgets.datewidget import FriendlyDatetimeDisplayWidget
from zope.formlib.textwidgets import BytesDisplayWidget
from waeup.kofa.students.interfaces import IStudentsUtils
from waeup.kofa.applicants.interfaces import (
    IApplicantRegisterUpdate, IApplicant, IApplicantEdit)
from waeup.kofa.applicants.browser import (ApplicantDisplayFormPage,
    ApplicantManageFormPage, ApplicantEditFormPage,
    ApplicantRegistrationPage, ApplicantAddFormPage,
    OnlinePaymentDisplayFormPage, ApplicationFeePaymentAddPage,
    OnlinePaymentBreadcrumb, ExportPDFPaymentSlipPage,
    ApplicantBaseDisplayFormPage)
from waeup.kofa.applicants.viewlets import (
    PaymentReceiptActionButton, PDFActionButton)
from waeup.kofa.applicants.pdf import PDFApplicationSlip
from kofacustom.nigeria.applicants.interfaces import (
    INigeriaPGApplicant, INigeriaUGApplicant,
    INigeriaPGApplicantEdit, INigeriaUGApplicantEdit,
    INigeriaApplicantOnlinePayment,IPUTMEApplicantEdit,
    UG_OMIT_DISPLAY_FIELDS,
    UG_OMIT_PDF_FIELDS,
    UG_OMIT_MANAGE_FIELDS,
    UG_OMIT_EDIT_FIELDS,
    PG_OMIT_DISPLAY_FIELDS,
    PG_OMIT_PDF_FIELDS,
    PG_OMIT_MANAGE_FIELDS,
    PG_OMIT_EDIT_FIELDS,
    PUTME_OMIT_DISPLAY_FIELDS,
    PUTME_OMIT_PDF_FIELDS,
    PUTME_OMIT_MANAGE_FIELDS,
    PUTME_OMIT_EDIT_FIELDS,
    PUTME_OMIT_RESULT_SLIP_FIELDS,
    PUDE_OMIT_DISPLAY_FIELDS,
    PUDE_OMIT_PDF_FIELDS,
    PUDE_OMIT_MANAGE_FIELDS,
    PUDE_OMIT_EDIT_FIELDS,
    PUDE_OMIT_RESULT_SLIP_FIELDS,
    )
from kofacustom.nigeria.interfaces import MessageFactory as _

class NigeriaOnlinePaymentBreadcrumb(OnlinePaymentBreadcrumb):
    """A breadcrumb for payments.
    """
    grok.context(INigeriaApplicantOnlinePayment)

class PDFActionButton(PDFActionButton):

    @property
    def text(self):
        if getattr(self.context, 'result_uploaded', False):
            return _('Download result slip')
        return _('Download application slip')

class NigeriaApplicantDisplayFormPage(ApplicantDisplayFormPage):
    """A display view for applicant data.
    """

    def _not_paid(self):
        return self.context.state in ('initialized', 'started',)

    @property
    def form_fields(self):
        if self.target is not None and self.target.startswith('pg'):
            form_fields = grok.AutoFields(INigeriaPGApplicant)
            for field in PG_OMIT_DISPLAY_FIELDS:
                form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('putme'):
            form_fields = grok.AutoFields(INigeriaUGApplicant)
            for field in PUTME_OMIT_DISPLAY_FIELDS:
                form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('pude'):
            form_fields = grok.AutoFields(INigeriaUGApplicant)
            for field in PUDE_OMIT_DISPLAY_FIELDS:
                form_fields = form_fields.omit(field)
        else:
            form_fields = grok.AutoFields(INigeriaUGApplicant)
            for field in UG_OMIT_DISPLAY_FIELDS:
                form_fields = form_fields.omit(field)
        #form_fields['perm_address'].custom_widget = BytesDisplayWidget
        form_fields['notice'].custom_widget = BytesDisplayWidget
        if not getattr(self.context, 'student_id'):
            form_fields = form_fields.omit('student_id')
        if not getattr(self.context, 'screening_score'):
            form_fields = form_fields.omit('screening_score')
        if not getattr(self.context, 'screening_venue') or self._not_paid():
            form_fields = form_fields.omit('screening_venue')
        if not getattr(self.context, 'screening_date') or self._not_paid():
            form_fields = form_fields.omit('screening_date')
        return form_fields

class NigeriaPDFApplicationSlip(PDFApplicationSlip):

    def _reduced_slip(self):
        return getattr(self.context, 'result_uploaded', False)

    @property
    def form_fields(self):
        if self.target is not None and self.target.startswith('pg'):
            form_fields = grok.AutoFields(INigeriaPGApplicant)
            for field in PG_OMIT_PDF_FIELDS:
                form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('putme'):
            form_fields = grok.AutoFields(INigeriaUGApplicant)
            if self._reduced_slip():
                for field in PUTME_OMIT_RESULT_SLIP_FIELDS:
                    form_fields = form_fields.omit(field)
            else:
                for field in PUTME_OMIT_PDF_FIELDS:
                    form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('pude'):
            form_fields = grok.AutoFields(INigeriaUGApplicant)
            if self._reduced_slip():
                for field in PUDE_OMIT_RESULT_SLIP_FIELDS:
                    form_fields = form_fields.omit(field)
            else:
                for field in PUDE_OMIT_PDF_FIELDS:
                    form_fields = form_fields.omit(field)
        else:
            form_fields = grok.AutoFields(INigeriaUGApplicant)
            for field in UG_OMIT_PDF_FIELDS:
                form_fields = form_fields.omit(field)
        if not getattr(self.context, 'student_id'):
            form_fields = form_fields.omit('student_id')
        if not getattr(self.context, 'screening_score'):
            form_fields = form_fields.omit('screening_score')
        if not getattr(self.context, 'screening_venue'):
            form_fields = form_fields.omit('screening_venue')
        if not getattr(self.context, 'screening_date'):
            form_fields = form_fields.omit('screening_date')
        return form_fields

class NigeriaApplicantManageFormPage(ApplicantManageFormPage):
    """A full edit view for applicant data.
    """
    
    @property
    def form_fields(self):
        if self.target is not None and self.target.startswith('pg'):
            form_fields = grok.AutoFields(INigeriaPGApplicant)
            for field in PG_OMIT_MANAGE_FIELDS:
                form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('putme'):
            form_fields = grok.AutoFields(INigeriaUGApplicant)
            for field in PUTME_OMIT_MANAGE_FIELDS:
                form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('pude'):
            form_fields = grok.AutoFields(INigeriaUGApplicant)
            for field in PUDE_OMIT_MANAGE_FIELDS:
                form_fields = form_fields.omit(field)
        else:
            form_fields = grok.AutoFields(INigeriaUGApplicant)
            for field in UG_OMIT_MANAGE_FIELDS:
                form_fields = form_fields.omit(field)
        form_fields['student_id'].for_display = True
        form_fields['applicant_id'].for_display = True
        return form_fields

class NigeriaApplicantEditFormPage(ApplicantEditFormPage):
    """An applicant-centered edit view for applicant data.
    """

    @property
    def form_fields(self):
        if self.target is not None and self.target.startswith('pg'):
            form_fields = grok.AutoFields(INigeriaPGApplicantEdit)
            for field in PG_OMIT_EDIT_FIELDS:
                form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('putme'):
            form_fields = grok.AutoFields(IPUTMEApplicantEdit)
            for field in PUTME_OMIT_EDIT_FIELDS:
                form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('pude'):
            form_fields = grok.AutoFields(INigeriaUGApplicantEdit)
            for field in PUDE_OMIT_EDIT_FIELDS:
                form_fields = form_fields.omit(field)
        else:
            form_fields = grok.AutoFields(INigeriaUGApplicantEdit)
            for field in UG_OMIT_EDIT_FIELDS:
                form_fields = form_fields.omit(field)
        form_fields['applicant_id'].for_display = True
        form_fields['reg_number'].for_display = True
        return form_fields

class NigeriaOnlinePaymentDisplayFormPage(OnlinePaymentDisplayFormPage):
    """ Page to view an online payment ticket
    """
    grok.context(INigeriaApplicantOnlinePayment)
    form_fields = grok.AutoFields(INigeriaApplicantOnlinePayment).omit('ac',
        'provider_amt', 'gateway_amt', 'thirdparty_amt', 'p_item')
    form_fields[
        'creation_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')
    form_fields[
        'payment_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')

class NigeriaExportPDFPaymentSlipPage(ExportPDFPaymentSlipPage):
    """Deliver a PDF slip of the context.
    """
    grok.context(INigeriaApplicantOnlinePayment)
    form_fields = grok.AutoFields(INigeriaApplicantOnlinePayment).omit(
        'ac', 'provider_amt', 'gateway_amt', 'thirdparty_amt', 'p_item')
    form_fields['creation_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')
    form_fields['payment_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')

class NigeriaApplicantRegistrationPage(ApplicantRegistrationPage):
    """Captcha'd registration page for applicants.
    """

    #def _redirect(self, email, password, applicant_id):
    #    # Forward email and credentials to landing page.
    #    self.redirect(self.url(self.context, 'registration_complete',
    #        data = dict(email=email, password=password,
    #        applicant_id=applicant_id)))
    #    return